<?php

namespace Teknisa\SM\Service;

use Teknisa\Libs\Util\ConnectionCustom;
use Doctrine\ORM\EntityManager;
use Teknisa\SM\Factory\IntegracaoOmFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\SM\Controller\BirtReport;
use Teknisa\Libs\Util\NewCode;
use Zeedhi\Framework\Session\Session;
use Teknisa\SM\Util\Procedure\FctSincronizarComposicao;
use DateTime;
use Teknisa\SM\Service\IntegracoesTeknisaService;
use Teknisa\SM\Util\Procedure\FctGerarParcelasApuracao;
use Teknisa\Libs\Util\DateUtil;
//use Teknisa\SM\Service\Sm00012000MedicaoFaturamentoService;
use Teknisa\Libs\Util\SMTP as SMTPService;

class IntegracaoOmService {

    const NOME_JOB_GERAR_PARCELA_MEDICOES_CALCULADAS = "SM - Gerar parcela das medições calculadas";

    protected $integracaoFactory;
    protected $environment;
    protected $newCode;
    protected $session;
    protected $connectioncustom;
    protected $entityManager;
    protected $connection;
    protected $integracoesTeknisaService;
    protected $smtpService;
    protected $emailDestinMedicaoAuto;
    protected $medicaoService;
    
    public function __construct(IntegracaoOmFactory $integracaoFactory, Environment $environment, Session $session, ConnectionCustom $connectioncustom, EntityManager $entityManager, IntegracoesTeknisaService $integracoesTeknisaService, $emailDestinMedicaoAuto, /*Sm00012000MedicaoFaturamentoService*/ $medicaoService) {
        $this->integracaoFactory         = $integracaoFactory;
        $this->environment               = $environment;
        $this->session                   = $session;
        $this->connectioncustom          = $connectioncustom;
        $this->entityManager             = $entityManager;
        $this->integracoesTeknisaService = $integracoesTeknisaService;
        $this->emailDestinMedicaoAuto    = $emailDestinMedicaoAuto;
        $this->medicaoService            = $medicaoService; // Teknisa\SM\Service\Sm00012000MedicaoFaturamentoService
    }

    /**
     * Monta o array da integração.
     * 
     * @param integer $nrseqorcamento
     * @param integer $nrorg
     * 
     * @return mixed[]
     */
    public function montarArrayIntegracao($nrseqorcamento, $nrorg, $periodoExtendido = null, $nrseqcontrato = null, $suspension = false) {
        
        $solicitante = $this->montarArraySolicitante();

        if (is_null($nrseqcontrato)) {
            $produtos = $this->montarArrayProdutosProposta($nrseqorcamento, $nrorg);

            return array(
                "products"  => $produtos,
                "requester" => $solicitante
            );
        }
        else {
            $cliente  = $this->montarArrayCliente($nrseqcontrato, $nrorg);
            $produtos = $this->montarArrayProdutosContrato($nrseqcontrato, $nrorg, $suspension);
            
            $periodoVigenciaContrato = $this->getPeriodoVigenciaContrato($nrseqcontrato, $nrorg);
            $initialDate    = $periodoVigenciaContrato["DTINIVIGENCONT"];
            $expirationDate = $periodoVigenciaContrato["DTFINVIGENCONT"];
            
            // $initialDate = new DateTime();
            // $initialDate = date_format($initialDate,'d/m/Y');
            
            $dataset = array(
                "client"           => $cliente,
                "products"         => $produtos,
                "requester"        => $solicitante,
                "initialDate"      => $initialDate,
                "expirationDate"   => $expirationDate,
                "isExtendContract" => $periodoExtendido,
                "contractCode"     => $nrseqcontrato,
                "isSuspension"     => $suspension
            );
            
            $dadosContratoPai = $this->getDadosContratoPai($nrseqcontrato, $nrorg);
            if ($dadosContratoPai && $dadosContratoPai["NRSEQCONTRATO"]) {
                $dataset["oldContractCode"] = $dadosContratoPai["NRSEQCONTRATO"];
                $dataset["oldInitialDate"] = $dadosContratoPai["DTINIVIGENCONT"];
                $dataset["oldExpirationDate"] = $dadosContratoPai["DTFINVIGENCONT"];
                $dataset["rescissionDate"] = $dadosContratoPai["DTFINVIGENCONT"];
                $dataset["rescissionJustification"] = $dadosContratoPai["DSJUSTIFCANCRENEG"];
            }
            
            return $dataset;
        }
    }
    
    /**
     * Monta a parte do cliente do array da integração.
     * 
     * @param integer $nrseqorcamento
     * @param integer $nrorg
     * 
     * @return mixed[]
     */
    private function montarArrayCliente($nrseqcontrato, $nrorg) {
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg
        );
        
        $dataset = $this->integracaoFactory->getDadosCliente($params);
        
        return array(
            "code"          => $dataset[0]["CDCLIENTE"],
            "operatorName"  => $dataset[0]["NOME_INTERLOCUTOR"],
            "operatorEmail" => $dataset[0]["EMAIL_INTERLOCUTOR"]
        );
    }
    
    /**
     * Monta a parte dos produtos do array da integração.
     * 
     * @param integer $nrseqorcamento
     * @param integer $nrorg
     * 
     * @return mixed[]
     */
    private function montarArrayProdutosProposta($nrseqorcamento, $nrorg) {
        $arrayProdutos = array();
        $produtos      = $this->getProdutosProposta($nrseqorcamento, $nrorg);
        
        foreach ($produtos as $produto) {
            $idelementoProduto = $produto["IDELEMENTO"];
            $productId = substr($idelementoProduto, 2);
            $arrayElementos = $this->montarArrayElementosProposta($nrseqorcamento, $idelementoProduto, $nrorg);
            
            $arrayProdutos[] = array(
                "productId" => $productId,
                "elements"  => $arrayElementos
            );
        }
        
        return $arrayProdutos;
    }

    private function montarArrayProdutosContrato($nrseqcontrato, $nrorg, $suspension = false) {
        $arrayProdutos = array();
        $produtos      = $this->getProdutosContrato($nrseqcontrato, $nrorg);
        
        foreach ($produtos as $produto) {
            $idelementoProduto  = $produto["IDELEMENTO"];
            $productId          = substr($idelementoProduto, 2);
            $cdtpmedidavariavel = $this->getCdtpMedidaVariavel($productId, $nrseqcontrato, $nrorg);
            $qtMinLicenses      = $this->getQuantidadeMinimaContrato($cdtpmedidavariavel, $nrorg, $nrseqcontrato, $idelementoProduto);
            $licenseValue       = $this->getLicenuso($nrseqcontrato, $idelementoProduto, $nrorg);
            $licenseMaintenance = $this->getLicenManut($nrseqcontrato, $idelementoProduto, $nrorg);
            $arrayElementos     = $this->montarArrayElementosContrato($nrseqcontrato, $idelementoProduto, $nrorg, $suspension);
            
            $isCreateProductLine = false;
            if (!is_null($licenseMaintenance) || !is_null($licenseValue)) {
                $isCreateProductLine = true;
            }

            $arrayProdutos[] = array(
                "productId"           => $productId,
                "elements"            => $arrayElementos,
                "licenseValue"        => $licenseValue,
                "licenseMaintenance"  => $licenseMaintenance,
                "qtMinLicenses"       => $qtMinLicenses,
                "contractualName"     => $produto["NMPROD"],
                "isCreateProductLine" => $isCreateProductLine
            );
        }
        
        // Adiciona nos parametros os produtos retirados do contrato
        $arrayProdRetirados = $this->getArrayProdutosRetiradosDoContrato($nrseqcontrato, $nrorg, $produtos);
        if(count($arrayProdRetirados) > 0)
            $arrayProdutos = array_merge($arrayProdutos, $arrayProdRetirados);

        return $arrayProdutos;
    }
    
    /**
     * Retorna a data de validade dos elementos no formato YYYY-MM-DD.
     * 
     * @param integer $nrseqorcamento
     * @param integer $nrorg
     * 
     * @return string
     */
    private function getPeriodoVigenciaContrato($nrseqcontrato, $nrorg) {
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg
        );
        $dataset = $this->integracaoFactory->getPeriodoVigenciaContrato($params);
        return $dataset[0];
    }

    private function getCdservorcaToQtMin($nrseqorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG"          => $nrorg
        );

        return $this->integracaoFactory->getCdservorcaToQtMin($params);
    }

    private function getLicenuso($nrseqcontrato, $idelemento, $nrorg) {
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg,
            "IDELEMENTO"    => $idelemento
        );

        $licenseValue = $this->integracaoFactory->getLicenuso($params);
        return (count($licenseValue) > 0) ? $licenseValue[0]["VRUNITARIO"] : null;
    }

    private function getLicenManut($nrseqcontrato, $idelemento, $nrorg) {
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg,
            "IDELEMENTO"    => $idelemento
        );

        $licenseValue = $this->integracaoFactory->getLicenManut($params);
        return (count($licenseValue) > 0) ? $licenseValue[0]["VRUNITARIO"] : null;
    }

    private function getQuantidadeMinimaContrato($cdtpmedidavariavel, $nrorg, $nrseqcontrato, $idelemento) {
        $params = array(
            "NRSEQCONTRATO"      => $nrseqcontrato,
            "CDTPMEDIDAVARIAVEL" => $cdtpmedidavariavel,
            "IDELEMENTORAIZ"     => $idelemento,
            "NRORG"              => $nrorg
        );
        $qtsMinLicensas = $this->integracaoFactory->getQuantidadeMinimaLicencaContrato($params);
        return (count($qtsMinLicensas) > 0) ? $qtsMinLicensas[0]["QTMEDIDAVARIAVEL"] : null;
    }

    private function getCdtpMedidaVariavel($productId, $nrseqcontrato, $nrorg) {
        $params = array(
            "ID"    => $productId,
            "NRORG" => $nrorg
        );
        $cdtpmedidavariavel = $this->integracaoFactory->getCdtpMedidaVariavelFromOm($params);
        return (count($cdtpmedidavariavel) > 0) ? $cdtpmedidavariavel[0]["CDTPMEDIDAVARIAVEL"] : null;
    }
    
    /**
     * Monta a parte dos elementos do array da integração.
     * 
     * @param integer $nrseqorcamento
     * @param string $idProduto
     * @param string $dataValidade
     * @param integer $nrorg
     * 
     * @return mixed[]
     */
    private function montarArrayElementosProposta($nrseqorcamento, $idProduto, $nrorg) {
        $arrayElementos = array();
        $menus = $this->getMenusDoProdutoProposta($nrseqorcamento, $nrorg, $idProduto);
        
        foreach ($menus as $menu) {
            $arrayElementos[] = array(
                "id" => substr($menu["IDELEMENTO"], strlen($idProduto) + 3),
            );
        }

        return $arrayElementos;
    }

    private function montarArrayElementosContrato($nrseqcontrato, $idProduto, $nrorg, $suspension = false) {
        $arrayElementos = array();
        $menus = $this->getMenusDoProdutoContrato($nrseqcontrato, $nrorg, $idProduto, $suspension);
        $menus = $this->getMenusDoProdutoContratoUnique($menus);
        
        foreach ($menus as $menu) {
            $arrayElementos[] = array(
                "id"             => substr($menu["IDELEMENTO"], strlen($idProduto) + 3),
                "isUpdate"       => ($menu["IS_UPDATE"] == 'S'),
                "expirationDate" => $menu["DTFINVIGENCONT"],
                "relatedElement" => ($menu["IDAPARECEPROPOSTA"] == "N")
            );
        }

        return $arrayElementos;
    }
    
    /**
     * Retorna os produtos da composição da proposta.
     * 
     * @param integer $nrseqorcamento
     * @param integer $nrorg
     * 
     * @return mixed[]
     */
    private function getProdutosProposta($nrseqorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG"          => $nrorg
        );
        
        return $this->integracaoFactory->getElementosTipoProdutoProposta($params);
    }

    /**
     * Retorna os produtos da composição do contrato.
     * 
     * @param integer $nrseqorcamento
     * @param integer $nrorg
     * 
     * @return mixed[]
     */
    private function getProdutosContrato($nrseqcontrato, $nrorg) {
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg
        );
        
        return $this->integracaoFactory->getElementosTipoProdutoContrato($params);
    }
    
    /**
     * Retorna os menus de último nível do produto da composição da proposta.
     * 
     * @param integer $nrseqorcamento
     * @param integer $nrorg
     * @param integer $idelemento
     * 
     * @return mixed[]
     */
    private function getMenusDoProdutoProposta($nrseqorcamento, $nrorg, $idelemento) {
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG"          => $nrorg,
            "IDELEMENTO"     => $idelemento
        );
        
        return $this->integracaoFactory->getElementosFilhosTipoMenuPorProdutoProposta($params);
    }

    private function getMenusDoProdutoContrato($nrseqcontrato, $nrorg, $idelemento, $suspension = false) {
        
        if ($suspension == true){
            $susp = 'S';
        }
        else{
           $susp = 'N'; 
        }
        
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "SUSPENSAO"     => $susp,
            "NRORG"         => $nrorg,
            "IDELEMENTO"    => $idelemento
        );
        
        return $this->integracaoFactory->getElementosFilhosTipoMenuPorProdutoContrato($params);
    }
    
    /**
     * Monta a parte do solicitante do array da integração.
     * 
     * @param integer $nrseqorcamento
     * @param integer $nrorg
     * 
     * @return mixed[]
     */
    private function montarArraySolicitante() {
        $cdoperador = $this->environment->getCdOperador();
        $nrorg      = $this->environment->getNrOrg();
        
        return array(
            "userId"         => $cdoperador ? $cdoperador : "000000000997",
            "organizationId" => $nrorg ? $nrorg : 20
        );
    }
    
    public function nrcomposicao(array $produto){
        try{

            $dados = array(
                "IDELEMENTO"       => "P_" . $produto["id"],
            );
            
            $result = $this->integracaoFactory->nrcomposicao($dados);
            
            return count($result) > 0 ? $result : null;
        }
        catch(\Exception $e){
            throw $e;
        }
    }

    public function validacomposicao(array $produto){
        try{

            $dados = array(
                "IDELEMENTO"       => "P_" . $produto["id"],
            );
            $result = $this->integracaoFactory->validacomposicao($dados);
            
            return count($result) > 0 ? $result : null;
            
        }
        catch(\Exception $e){
            throw $e;
        }
    }

    public function sincronizarComp($nrcomp, $nrorg){

        try {
            $entityManagerConnection = $this->connectioncustom->updateVpdConnection($nrorg);

            if($entityManagerConnection !== false){
                $this->entityManager = $entityManagerConnection;
            }
            $this->connection = $this->entityManager->getConnection();
            $this->connection->beginTransaction();
            $procedure = new FctSincronizarComposicao($this->connection);

            foreach($nrcomp as $nr){
                $procedure->executar($nr["NRCOMPSERVICO"], $nrorg);
            }

            $this->connection->commit();
            return "ola";
        }catch(\Exception $e){
            throw $e;
        }

    }

    public function validaIntegracaoOm(array $produto, array $elementos, $nrorg, $cdoperador, $atualizarcomp = false){
        try {
            $entityManagerConnection = $this->connectioncustom->updateVpdConnection($nrorg);
                
            if($entityManagerConnection !== false){
                $this->entityManager = $entityManagerConnection;
            }
                
            $this->connection = $this->entityManager->getConnection();
            $this->newCode = new NewCode($this->connection, "NOVO_CODIGO_SC");
            $this->integracaoFactory = new IntegracaoOmFactory($this->entityManager);
            
            $this->connection->beginTransaction();
            try {
                $this->sincronizarElementos($produto, $elementos, $nrorg, $cdoperador);
                if($atualizarcomp === true){
                    $this->sincronizarComposicoesServico($produto, $nrorg);

                }
                $this->connection->commit();
            }
            catch (\Exception $e) {
                $this->connection->rollback();
                throw $e;
            }
        }
        catch(\Exception $e){
            throw $e;
        }
    }
    
    /**
     * Sincroniza os elementos do OM com o SM.
     * 
     * @param mixed[] $produto
     * @param mixed[] $elementos
     * @param integer $nrorg
     * @param string  $cdoperador
     */
    public function sincronizarElementos(array $produto, array $elementos, $nrorg, $cdoperador) {
        $elementos = $this->getElementosParaSalvar($produto, $elementos, $nrorg, $cdoperador);
        $existeProduto = $this->verificarExistenciaProduto($produto["id"], $nrorg);
        
        if ($existeProduto) {
            $prod = array_shift($elementos);
            
            $elementosRequisicao = array();
            foreach ($elementos as $elemento) {
                $elementosRequisicao[] = $elemento["IDELEMENTO"];
            }
            
            $elementosInseridos = $this->getElementosDoProduto($produto["id"], $nrorg);
            $codElementosInseridos = array();
            
            foreach ($elementosInseridos as $elemento) {
                $codElementosInseridos[] = $elemento["IDELEMENTO"];
            }
            
            $codElementosParaInserir   = array_diff($elementosRequisicao, $codElementosInseridos);
            $codElementosParaAtualizar = array_intersect($elementosRequisicao, $codElementosInseridos);
            $codElementosParaExcluir   = array_diff($codElementosInseridos, $elementosRequisicao);

            $this->excluirElementos($codElementosParaExcluir, $nrorg);
            
            $this->prepararAtualizarElemento($prod);
            
            foreach ($elementos as $elemento) {
                if (in_array($elemento["IDELEMENTO"], $codElementosParaInserir)) {
                    $this->prepararInserirElemento($elemento, $nrorg);
                }
                else if (in_array($elemento["IDELEMENTO"], $codElementosParaAtualizar)) {
                    $this->prepararAtualizarElemento($elemento);
                }
            }
        }
        else {
            foreach ($elementos as $elemento) {
                $this->prepararInserirElemento($elemento, $nrorg);
            }
        }
    }
    
    /**
     * Monta o array com todos os dados dos elementos para a sincronização com a tabela.
     * 
     * @param mixed[] $produto
     * @param mixed[] $elementos
     * @param integer $nrorg
     * @param string  $cdoperador
     * 
     * @return mixed[]
     */
    private function getElementosParaSalvar(array $produto, array $elementos, $nrorg, $cdoperador) {
        $dados = array();
        
        $dados[] = array(
            "CDSERVORCA"       => null,
            "NMSERVORCA"       => $produto["name"],
            "IDELEMENTO"       => "P_" . $produto["id"],
            "IDELEMENTOPAI"    => null,
            "IDTIPO"           => "E",
            "IDTIPOELEMENTO"   => "P",
            "IDATIVO"          => "S",
            "NRORG"            => $nrorg,
            "CDOPERINCLUSAO"   => $cdoperador,
            "IDORDENAELEMENTO" => 0
        );
        
        foreach ($elementos as $chave => $elemento) {
            $dados[] = array(
                "CDSERVORCA"       => null,
                "NMSERVORCA"       => $elemento["name"],
                "IDELEMENTO"       => "P_" . $produto["id"] . "_M_" . $elemento["id"],
                "IDELEMENTOPAI"    => ($elemento["parent"]) ? ("P_" . $produto["id"] . "_M_" . $elemento["parent"]) : ("P_" . $produto["id"]),
                "IDTIPO"           => "E",
                "IDTIPOELEMENTO"   => "M",
                "IDATIVO"          => "S",
                "NRORG"            => $nrorg,
                "CDOPERINCLUSAO"   => $cdoperador,
                "IDORDENAELEMENTO" => $chave + 1
            );
        }
        
        return $dados;
    }
    
    /**
     * Verifica se o produto existe na base de dados.
     * 
     * @param integer $idProduto
     * @param integer $nrorg
     * 
     * @return boolean
     */
    private function verificarExistenciaProduto($idProduto, $nrorg) {
        $params = array(
            "NRORG"      => $nrorg,
            "IDELEMENTO" => "P_" . $idProduto
        );
        
        $dataset = $this->integracaoFactory->verificarExistenciaProduto($params);
        return ((int)$dataset[0]["EXISTE"] > 0);
    }
    
    /**
     * Retorna os elementos do produto informado.
     * 
     * @param integer $idProduto
     * @param integer $nrorg
     * 
     * @return mixed[]
     */
    private function getElementosDoProduto($idProduto, $nrorg) {
        $params = array(
            "NRORG"      => $nrorg,
            "IDELEMENTO" => "P_" . $idProduto
        );
        
        return $this->integracaoFactory->getElementosDoProduto($params);
    }
    
    /**
     * Prepara o elemento para a inserção, colocando o NovoCodigo.
     * 
     * @param mixed[] $elemento
     * @param integer $nrorg
     */
    private function prepararInserirElemento($elemento, $nrorg) {
        $cdservorca = $this->newCode->getCodeByTableName("FCT_SERVICOORCA", $nrorg, 1, 10, true);
        $elemento["CDSERVORCA"] = $cdservorca;
        
        $this->integracaoFactory->inserirElemento($elemento);
    }
    
    /**
     * Prepara o elemento para sua atualização.
     * 
     * @param mixed[] $elemento
     */
    private function prepararAtualizarElemento($elemento) {
        $params = array(
            "NMSERVORCA"       => $elemento["NMSERVORCA"],
            "IDELEMENTOPAI"    => $elemento["IDELEMENTOPAI"],
            "IDORDENAELEMENTO" => $elemento["IDORDENAELEMENTO"],
            "IDELEMENTO"       => $elemento["IDELEMENTO"],
            "NRORG"            => $elemento["NRORG"],
            "CDOPERULTATU"     => $this->environment->getCdOperador(),
            "NRORGULTATU"      => $this->environment->getNrOrg()
        );
        
        $this->integracaoFactory->atualizarElemento($params);
    }
    
    /**
     * Exclui os elementos.
     * 
     * @param integer[] $codElementos
     * @param integer   $nrorg
     */
    private function excluirElementos(array $codElementos, $nrorg) {
        if (count($codElementos) > 0) {
            
            foreach ($codElementos as &$cod) {
                $cod = "'" . $cod . "'";
            }
            
            $codElementos = implode(",", $codElementos);
            $this->integracaoFactory->excluirElementos($codElementos, $nrorg);
         
            $retornoComposicao = $this->integracaoFactory->composicaoServicoElementos($codElementos, $nrorg);

            $procedure = new FctSincronizarComposicao($this->connection);
            foreach ($retornoComposicao as $comp) {
                $procedure->executar($comp["NRCOMPSERVICO"], $nrorg);
            }
        }
    }
    
    /**
     * Verifica se o sistema utiliza a integração com o OM.
     * 
     * @param integer $nrorg
     * 
     * @return boolean
     */
    public function verificarUtilizaIntegracao($nrorg) {
        $params = array(
            "NRORG" => $nrorg
        );
        
        $dataset = $this->integracaoFactory->verificarUtilizaIntegracao($params);
        return ($dataset[0]["IDINTEGRACAOOM"] == "S");
    }
    
    /**
     * Retorna a URL de integração com o OM.
     * 
     * @param integer $nrorg
     * 
     * @return string
     */
    public function getUrlIntegracao($nrorg) {
        $params = array(
            "NRORG" => $nrorg
        );
        
        $dataset = $this->integracaoFactory->getUrlIntegracao($params);
        return $dataset[0]["URLINTEGRACAOOM"];
    }
    
    /**
     * Atualiza as composições de serviço para retirar elementos que não existem mais.
     */
    public function sincronizarComposicoesServico(array $produto, $nrorg) {
        $composicoes = $this->getComposicoesPorProduto($produto["id"], $nrorg);
        $procedure = new FctSincronizarComposicao($this->connection);
        foreach ($composicoes as $comp) {
            $procedure->executar($comp, $nrorg);
        }


    }
    
    /**
     * Retorna o código das composições de serviço que contém o elemento informado.
     */
    public function getComposicoesPorProduto($idProduto, $nrorg) {
        $params = array(
            "IDELEMENTO" => "P_" . $idProduto,
            "NRORG"      => $nrorg
        );

        $dataset = $this->integracaoFactory->getComposicoesPorProduto($params);
        $composicoes = array();
        
        foreach ($dataset as $row) {
            $composicoes[] = $row["NRCOMPSERVICO"];
        }
        
        return $composicoes;
    }
    
    public function integracaoOM($nrseqorcamento, $nrorg, $idutilizacomp, $periodoExtendido, $nrseqcontrato, $suspension = false) {
        if ($idutilizacomp == 'S'){
            $utilizaIntegracao = $this->verificarUtilizaIntegracao($nrorg);
            if ($utilizaIntegracao) {
                try {
                    //Envia os Elementos para o OM
                    $url   = $this->getUrlIntegracao($nrorg);
                    $dados = $this->montarArrayIntegracao($nrseqorcamento, $nrorg, $periodoExtendido, $nrseqcontrato, $suspension);
                    
                    if(!empty($dados) && array_key_exists("products", $dados)) {
                        $this->salvarRescisaoContratoIntegracaoOM($dados,$nrseqcontrato, $nrorg);
                        $retorno = $this->integracoesTeknisaService->enviarDadosIntegracaoOM($url, "/createOrganizationByIntegration/save", $dados);
                        if (!isset($retorno["clientData"]["organizationId"])) {
                            throw new \Exception("Falha na integração com o módulo OM.");
                        }
                    }
                } catch (\Exception $e) {
                    $errorTrigger = strpos($e->getMessage(), "USR_BLOCK_DELETE_ELEMENTO_ORG");
                    if($errorTrigger === false)
                        throw $e;
                    else 
                        throw new \Exception("Operação bloqueda!<br><br>A Exclusão dos elementos por organização não é permitida. Favor entrar em contato com a Teknisa nos seguintes setores:<br><br> - Qualidade<br> - Controles");
                }  
            }
        }
    }
    
    /**
     * Realiza a apuração automática das medições de faturamento.
     */
    public function apurarMedicoesAutomatico($nrorg, $medicao = null) {
        if (empty($medicao)) {
            $tipoLog = '1';
            /* Arruma as conexões */
            $entityManagerConnection = $this->connectioncustom->updateVpdConnection($nrorg);
            if($entityManagerConnection !== false)
                $this->entityManager = $entityManagerConnection;
                
            $this->connection = $this->entityManager->getConnection();
            $this->newCode    = new NewCode($this->connection, "NOVO_CODIGO_SC");
            
            $this->excluirLogMedicoesAutoHoje($tipoLog, $nrorg);
        }
        else {
            $tipoLog = '2';
        }
        
        try {
            $operador = '1';
            
            if (empty($medicao)) {
                echo "Buscando as medições de faturamento a serem apuradas... \n\n";
                $medicoes = $this->buscaMedicoesFaturaAuto($nrorg, '3');
            }
            else {
                $medicoes = array($medicao);
            }
            
            if (is_array($medicoes) && count($medicoes) > 0) {
                $enviaEmail = 'N';
                foreach($medicoes as $med) {
                    $nrmedicaofatura = $med['NRMEDICAOFATURA'];
                    $nrseqcontrato   = $med['NRSEQCONTRATO'];
                
                    $apurar = $this->gerarApuracaoFatura($nrmedicaofatura, $nrseqcontrato, $nrorg, $operador);
                    if ($apurar == 'OK') {
                        $parcelaGerada = $this->buscaParcelasGeradasAuto($nrmedicaofatura, $nrorg);
                        if ($parcelaGerada != null){
                            $nrfaturacont = $parcelaGerada['NRFATURACONT'];
                            $log          = 'Log: Parcela gerada com sucesso!'; 
                            $this->gravaLogApuraMedAuto($nrmedicaofatura, $nrseqcontrato, $nrfaturacont, $log, $nrorg, $operador, $tipoLog);
                        }
                    } else {
                        $enviaEmail   = 'S';
                        $nrfaturacont = null;
                        $log          = 'Log: '.$apurar; 
                        $this->gravaLogApuraMedAuto($nrmedicaofatura, $nrseqcontrato, $nrfaturacont, $log, $nrorg, $operador, $tipoLog);   
                    }
                }
                
                $txtLog = '';
                
                $nrmedicaoLog = null;
                if (!empty($medicao)) {
                    $nrmedicaoLog = $medicao["NRMEDICAOFATURA"];
                }
                
                $logs = $this->verificaLogApuraMedAuto($nrorg, $tipoLog, $nrmedicaoLog);
                
                if ($logs != null) {
                    foreach($logs as $log) {
                        $txtLog = $txtLog.'Contrato: '.$log['NRCONTRATOFCT']."\n".'Cliente: '.$log['NMSECUNDARIPARC']."\n".'Medicao: '.$log['NRMEDICAOFATURA']."\n".'Competencia: '.$log['DTINICOMPET'].' - '.$log['DTFINCOMPET']."\n".'Fatura Gerada: '.$log['NRFATURACONT']."\n".'Valor: '.$log['VRVENCIMENTO']."\n"."\n"."Data do Log: ".$log['DTINCLUSAO']."\n"."\n".$log['TXLOG']."\n"."\n".'/----------------///----------------///----------------///----------------/'."\n"."\n"."\n"; 
                        echo $txtLog;
                        
                        if ($log['TXLOG'] <> 'Log: Parcela gerada com sucesso!')
                            $txtLogErr = $txtLog.'Contrato: '.$log['NRCONTRATOFCT'].chr(10).'Cliente: '.$log['NMSECUNDARIPARC'].chr(10).'Medicao: '.$log['NRMEDICAOFATURA'].chr(10).'Competencia: '.$log['DTINICOMPET'].' - '.$log['DTFINCOMPET'].chr(10).'Fatura Gerada: '.$log['NRFATURACONT'].chr(10).'Valor: '.$log['VRVENCIMENTO'].chr(10).chr(10)."Data do Log: ".$log['DTINCLUSAO'].chr(10).chr(10).$log['TXLOG'].chr(10).chr(10).'/----------------///----------------///----------------///----------------/'.chr(10).chr(10).chr(10);
                    }
                    
                    if ($enviaEmail == 'S') {
                        $conteudoArq = $txtLogErr;
                        $nomeArquivo = 'Log_Medicao_Automatica_'.date("d_m_Y").'.txt';
                        $arquivo = sys_get_temp_dir().DIRECTORY_SEPARATOR.$nomeArquivo;
                        
                        file_put_contents($arquivo, $conteudoArq);
                        $this->gerarEmailLog($nrorg, $arquivo);
                    }
                }
            } else {
                $nrmedicaofatura = null;
                $nrseqcontrato   = null;
                $nrfaturacont    = null;
                
                $log = "Não foram encontradas medições de faturamento a serem apuradas.";
                echo $log . "\n";
                $this->gravaLogApuraMedAuto($nrmedicaofatura, $nrseqcontrato, $nrfaturacont, $log, $nrorg, $operador, $tipoLog);  
                
            }
        } catch(\Exception $e){
            echo $e->getMessage();
            throw $e;
        }
    }
    
    /**
     * Retorna as medições para o faturamento automático.
     */
    public function buscaMedicoesFaturaAuto($nrorg, $idsituacao) {
        $params = array(
            "IDSITUACAO" => $idsituacao,
            "NRORG"      => $nrorg
        );

        return $this->integracaoFactory->buscaMedicoesFaturaAuto($params);
    }
    
    
    /**
     * Retorna os dados das propostas que foram geradas.
     */
    public function buscaParcelasGeradasAuto($nrmedicaofatura, $nrorg) {
        try { 
            $params = array(
                "NRMEDICAOFATURA" => $nrmedicaofatura,
                "NRORG"           => $nrorg
            );
    
            $dataset = $this->integracaoFactory->buscaParcelasGeradasAuto($params);
            $retorno = (count($dataset) > 0) ? $dataset[0] : null;
            return $retorno;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Executa a procedure "FCT_GERAR_PARCELAS_APURACAO".
     */
    public function gerarApuracaoFatura($nrmedicaofatura, $nrseqcontrato, $nrorg, $operador) {
        try {
            $nrseqcontratoAux   = '_'.$nrseqcontrato.'_';
            $nrmedicaofaturaAux = '_'.$nrmedicaofatura.'_';
            
            $procedure = new FctGerarParcelasApuracao($this->connection);
            $procedure->executar($nrseqcontratoAux, $nrmedicaofaturaAux, $nrorg, $operador);
            
            return 'OK';
        } catch(\Exception $e){
            return $e->getMessage();
        }
    }
    
    
    /**
     * Retorna os dados das propostas que foram geradas.
     */
    public function gravaLogApuraMedAuto($nrmedicaofatura, $nrseqcontrato, $nrfaturacont, $log, $nrorg, $operador, $idtipo) {
        try {
            $nrlogmedicaoauto = $this->newCode->getCodeByTableName("FCT_LOG_APURAMEDAUTO", $nrorg, 1, 10, true);
            $params = array(
                "NRLOGMEDICAOAUTO" => $nrlogmedicaoauto, 
                "NRMEDICAOFATURA"  => $nrmedicaofatura, 
                "NRSEQCONTRATO"    => $nrseqcontrato, 
                "NRFATURACONT"     => $nrfaturacont, 
                "TXLOG"            => $log,
                "NRORG"            => $nrorg,
                "CDOPERADOR"       => $operador,
                "IDTIPO"           => $idtipo
            );
    
            $this->integracaoFactory->gravaLogApuraMedAuto($params);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    
     /**
     * Verifica os logs gerados para gravar no arquivo.
     */
    public function verificaLogApuraMedAuto($nrorg, $idtipo = '1', $nrmedicaofatura = null) {
        try {
            $params = array(
                "NRORG"           => $nrorg,
                "IDTIPO"          => $idtipo,
                "NRMEDICAOFATURA" => $nrmedicaofatura
            );
            
            $retorno = $this->integracaoFactory->verificaLogApuraMedAuto($params);
            return $retorno;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    
    public function gerarEmailLog($nrorg, $arquivo){
        /* -- Configuração dos daddos de Host do E-mail -- */
        $dadosConfig = $this->getConfiguracoesEmail($nrorg);
        
        $params["host"]       = $dadosConfig["DSHOSTEMAIL"];
        $params["username"]   = $dadosConfig["DSUSUARIOEMAIL"];
        $params["password"]   = $dadosConfig["DSSENHAEMAIL"];
        $params["port"]       = $dadosConfig["DSPORTAEMAIL"];
        
        if ($dadosConfig["IDSMTPSECURE"] != null){
                    
            $params["smtpSecure"] = $dadosConfig["IDSMTPSECURE"];
            $params["smtpAuth"] = true;
        }

        $this->smtpService  = new SMTPService($params);
        
        /* -- Dados para envio do E-mail -- */
        
        $destinatarios = $this->emailDestinMedicaoAuto;
        $destinatarios = str_replace(" ", "", $destinatarios);
        $destinatarios = explode(";", $destinatarios);

        $remetente = 'sac@teknisa.com';
        $assunto   = "SM - Medição Automática: Log";
        $corpo     = "Prezados,<br><br> Ocorreram erros na geração automática das parcelas de faturamento.<br><br> Favor verificar o log em anexo e entrar em contato com o administrador do módulo SM."; 
            
        /* -- Envio do E-mail -- */
        $this->smtpService->setFrom($remetente);
        $this->smtpService->setFromName($remetente);
        $this->smtpService->setSubject($assunto);
        $this->smtpService->addAttachment($arquivo);
        $this->smtpService->sendMail($corpo, $destinatarios);
    }
    
    public function getConfiguracoesEmail($nrorg) {
        $params = array(
            "NRORG" => $nrorg
        );
        
        $retorno =  $this->integracaoFactory->getConfiguracoesEmail($params);
        return $retorno[0];
    }
    /**
     * Chama a função executarCurl para desabilitar no OM os elementos do cliente
    **/
    public function desabilitarElementosNoOm(array $params, $nrorg) {
        $url = $this->getUrlIntegracao($nrorg);
        $result = $this->integracoesTeknisaService->enviarDadosIntegracaoOM($url, "/removeOrganizationProductElements/save", $params);
    }
    
    /**
     * Retira os elementos duplicados do array de menus
    **/
    public function getMenusDoProdutoContratoUnique(array $menus) {
        $newArrayMenus = array();
        
        foreach($menus as $menu) {
            
            if($menu["IS_UPDATE"] == 'S')
                array_push($newArrayMenus, $menu);
        }
        
        foreach($menus as $menu) {
            if($menu["IS_UPDATE"] == 'N') {
                $exists = false;
                foreach($newArrayMenus as &$newMenu) {
                    if($menu["IDELEMENTO"] == $newMenu["IDELEMENTO"]) {
                        
                        $dtFinVigenMenu = DateUtil::getDataDeString($menu["DTFINVIGENCONT"], DateUtil::FORMATO_BRASILEIRO);
                        $dtFinVigenNewMenu = DateUtil::getDataDeString($newMenu["DTFINVIGENCONT"], DateUtil::FORMATO_BRASILEIRO);
                        
                        if($dtFinVigenMenu >= $dtFinVigenNewMenu) {
                           $newMenu["DTFINVIGENCONT"] = $menu["DTFINVIGENCONT"];
                        }
                        
                        $exists = true;
                        break;
                    }
                }
                unset($newMenu);
                
                if(!$exists)
                    array_push($newArrayMenus, $menu);
            }
        }
        
        return $newArrayMenus;
    }
    
    /**
     * Retorna os dados do contrato pai
     */
    public function getDadosContratoPai($nrseqcontrato, $nrorg) {
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg
        );
        
        $dataset = $this->integracaoFactory->getDadosContratoPai($params);
        return (count($dataset) > 0) ? $dataset[0] : null;
    }
    
    /**
     * Retorna um array de protudos retirados dos contratos renegociados
    **/
    public function getArrayProdutosRetiradosDoContrato($nrseqcontrato, $nrorg, $produtos) {
        $arrayProdRetirados  = array();
        $dadosContratoPai = $this->getDadosContratoPai($nrseqcontrato, $nrorg);
        if ($dadosContratoPai && $dadosContratoPai["NRSEQCONTRATO"]) {
            
            $nrseqcontratopai = $dadosContratoPai["NRSEQCONTRATO"];
            $produtosRetirados = $this->getProdutosRetiradosDoContrato($nrseqcontratopai, $nrseqcontrato, $nrorg);
            foreach($produtosRetirados as $produto) {
                $idelementoProduto = $produto["IDELEMENTO"];
                $arrayProdRetirados[] = array(
                    "productId"           => substr($idelementoProduto, 2),
                    "elements"            => array(),
                    "licenseValue"        => 0,
                    "licenseMaintenance"  => 0,
                    "qtMinLicenses"       => 0,
                    "contractualName"     => $produto["NMPROD"],
                    "isCreateProductLine" => false
                );
            }
        }
        
        return $arrayProdRetirados;
    }
    
    /**
     * Busca os contratos que estão presentes apenas no contrato antigo.
    **/
    public function getProdutosRetiradosDoContrato($nrseqcontratopai, $nrseqcontrato, $nrorg) {
        $params = array(
            "NRSEQCONTRATO"    => $nrseqcontrato,
            "NRSEQCONTRATOPAI" => $nrseqcontratopai,
            "NRORG"            => $nrorg
        );
        
        return $this->integracaoFactory->getProdutosRetiradosDoContrato($params);
    }
    
    
    public function getCountMedicoesSemColetas($nrorg, $qtdiasAnteFim) {
        $params = array(
            "QTDIASANTEFIM" => $qtdiasAnteFim,
            "NRORG"         => $nrorg,
        );
        
        $dataset = $this->integracaoFactory->getCountMedicoesSemColetas($params);
        return (int)$dataset[0]["QTD"];
    }
    
    public function apurarMedicoesQuantidadeContratual($nrorg) {
        /* Arruma as conexões */
        $entityManagerConnection = $this->connectioncustom->updateVpdConnection($nrorg);
        if($entityManagerConnection !== false)
            $this->entityManager = $entityManagerConnection;
            
        $this->connection = $this->entityManager->getConnection();
        $this->newCode    = new NewCode($this->connection, "NOVO_CODIGO_SC");
        
        $this->excluirLogMedicoesAutoHoje('2', $nrorg);
        
        try {
            echo "\n Buscando as medições de faturamento pendentes que serão calculadas com a quantidade contratual... \n\n";
            
            $medicoes = $this->buscaMedicoesFaturaAuto($nrorg, '1');
            if (is_array($medicoes) && count($medicoes) > 0) {
                foreach ($medicoes as $medicao) {
                    $nrmedicaofatura = $medicao["NRMEDICAOFATURA"];
                    $nrseqcontrato = $medicao["NRSEQCONTRATO"];
                    
                    $this->connection->beginTransaction();
                    try {
                        echo "Calculando a medição $nrmedicaofatura \n";
                        $this->medicaoService->recalcularItensMedicao($nrmedicaofatura, $nrseqcontrato, $nrorg);
                        echo "Fim do cálculo da medição $nrmedicaofatura \n\n";
                    }
                    catch (\Exception $e) {
                        $this->connection->rollback();
                        echo "ERRO ao calcular a medição $nrmedicaofatura:" . $e->getMessage() . "\n";
                        break;
                    }
                    
                    try {
                        echo "Gerando a parcela da medição $nrmedicaofatura \n";
                        $this->apurarMedicoesAutomatico($nrorg, $medicao);
                        echo "Fim da geração de parcela da medição $nrmedicaofatura \n\n";
                        
                        $this->connection->commit();
                    }
                    catch (\Exception $e) {
                        $this->connection->rollback();
                        echo "ERRO ao gerar parcela da medição $nrmedicaofatura:" . $e->getMessage() . "\n";
                    }
                }
                
                return true;
            }
            else {
                echo "Não há medições pendentes a serem calculadas.\n";
                return false;
            }
        }
        catch (\Exception $e) {
            echo $e->getMessage();
            throw $e;
        }
    }
    
    public function excluirLogMedicoesAutoHoje($idtipo, $nrorg) {
        $params = array(
            "IDTIPO" => $idtipo,
            "NRORG"  => $nrorg
        );
        
        $this->integracaoFactory->excluirLogMedicoesAuto($params);
    }
    
    public function salvarRescisaoContratoIntegracaoOM($dados, $nrseqcontrato, $nrorg, $rescisao = false) {
        try{
            $dados = json_encode($dados);
            $params = array(
                "DADOS"         => $dados,
                "NRSEQCONTRATO" => $nrseqcontrato,
                "NRORG"         => $nrorg
            );
            if($rescisao){
                $this->integracaoFactory->salvarRescisaoContratoIntegracaoOM($params);
            }
            else{
                $this->integracaoFactory->salvarAprovacaoContratoIntegracaoOM($params);
            }
        }
        catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Função para atualizar o status de execução da do Script no Jenkins
     * @param int $nrorg
     * @param string $idMomentoExec Define o momento que está sendo executada a funcão, sendo:
     * - "I" - Quando a função é iniciada.
     * - "F" - Quando é finalizada.
     * - "E" - Quando a função caiu em uma exceção.
     * @param mixed[] $dadosAnteriores
     * @param string $msgErro
     * 
     * @return mixed[]|null
     */
    public function alteraStatusExecucaoScript($nrorg, $idMomentoExec, $dadosAnteriores, $msgErro) {
        $this->connection = $this->entityManager->getConnection();
        $this->newCode    = new NewCode($this->connection, "NOVO_CODIGO_SC");
    
        $retorno = null;
        if ($idMomentoExec == 'I') {

            $dados = $this->integracaoFactory->dadosJobStatus();

            if (is_array($dados) && count($dados) > 0) {
                
                $retorno = $dados;
                
                $params = array(
                    "DTSUCESSO" => $dados[0]['DTSUCESSO'],
                    "DTERRO" => 'DTATUAL',
                    "DSERRO" => 'Erro na apuração automática de medições: A execução da Job foi interrompida.',
                    "NMJOB" => self::NOME_JOB_GERAR_PARCELA_MEDICOES_CALCULADAS
                );

                $this->integracaoFactory->updateDadosJobStatus($params); 
            }
            else {

                $id = $this->newCode->getCodeByTableName("JENKINS_JOB_STATUS", $nrorg, 1, 10, true);
                    
                $params = array(
                    "ID" => $id,
                    "NMJOB" => self::NOME_JOB_GERAR_PARCELA_MEDICOES_CALCULADAS,
                    "DSERRO" => 'Erro na apuração automática de medições: A execução da Job foi interrompida.',
                    "NRORG" => $nrorg
                );

                $this->integracaoFactory->insertDadosJobStatus($params); 
            }
        }
        else if ($idMomentoExec == 'F') {

            if (is_array($dadosAnteriores) && count($dadosAnteriores) > 0) {

                $dterro = $dadosAnteriores[0]['DTERRO'];
                $dserro = $dadosAnteriores[0]['DSERRO'];
            }
            else {

                $dterro = null;
                $dserro = null;
            }

            $params = array(
                "DTSUCESSO" => 'DTATUAL',
                "DTERRO" => $dterro,
                "DSERRO" => $dserro,
                "NMJOB" => self::NOME_JOB_GERAR_PARCELA_MEDICOES_CALCULADAS
            );

            $this->integracaoFactory->updateDadosJobStatus($params);
        }
        /*$idMomentoExec == 'E'*/
        else {
            
            if (is_array($dadosAnteriores) && count($dadosAnteriores) > 0) {

                $dtsucesso = $dadosAnteriores[0]['DTSUCESSO'];
            }
            else {

                $dtsucesso = null;
            }

            $params = array(
                "DTSUCESSO" => $dtsucesso,
                "DTERRO" => 'DTATUAL',
                "DSERRO" => $msgErro,
                "NMJOB" => self::NOME_JOB_GERAR_PARCELA_MEDICOES_CALCULADAS
            );

            $this->integracaoFactory->updateDadosJobStatus($params);
        }

        return $retorno;
    }
    
    /*Função para gravar os erros de medições sem coleta*/
    public function gravaErrosColetaOM($nrorg, $qtDiasAntFimMedicao) {
        $this->connection = $this->entityManager->getConnection();
        $this->newCode    = new NewCode($this->connection, "NOVO_CODIGO_SC");

        /*Busca as medições sem coleta.*/
        $params = array(
            "QTDIASANTEFIM" => $qtDiasAntFimMedicao
        );

        $medicoes = $this->integracaoFactory->getMedicoesSemColetas($params);
        echo "\nSalvando logs na tabela ERROS_COLETA...\n";
        foreach ($medicoes as $medicao) {
            
            /* Busca os scripts sql e php das coletas para inserção na tabela 'ERROS_COLETA' */

            $scriptCountPHP = null;
            $scriptLogPHP = null;
            $scriptCountSQL = null;
            $scriptLogSQL = null; 

            $paramsScript = array(
                "PRODUCT_ID"     => $medicao['PRODUCT_ID'],
                "CDTPMEDIDASERV" => $medicao['CDTPMEDIDAVARIAVEL']
            );

            $scripts = $this->integracaoFactory->getScriptsColetas($paramsScript);
            
            foreach ($scripts as $script) {
                if ($script['IDTPSCRIPT'] == 'PHP') {
                    $scriptCountPHP = $script['PHP_SCRIPT_COUNT'];
                    $scriptLogPHP = $script['PHP_SCRIPT_LOG'];
                }
                else {
                    $scriptCountSQL = $script['PHP_SCRIPT_COUNT'];
                    $scriptLogSQL = $script['PHP_SCRIPT_LOG'];
                }
            }

            /* Verifica se já existe um registro na tabela 'ERROS_COLETA' */

            $paramsValida = array(
                "NRSEQCONTRATO"  => $medicao['NRSEQCONTRATO'],
                "CDSERVORCA"     => $medicao['CDSERVORCA'],
                "CDTPMEDIDASERV" => $medicao['CDTPMEDIDAVARIAVEL'],
                "PRODUCT_ID"     => $medicao['NREXTERNAL_PRODUCT_ID'],
                "DTINICOMPET"    => $medicao['DTINICOMPET'],
                "DTFINCOMPET"    => $medicao['DTFINCOMPET']
            );
            
            $valida = $this->integracaoFactory->validaErrosColetaOM($paramsValida);

            /* Se não existir um registro, será inserido a linha com o erro de coleta. Se Existir, será updatada a linha com o erro de coleta */

            if ($valida == 0) {
                $id = $this->newCode->getCodeByTableName("ERROS_COLETA", $nrorg, 1, 10, true);
                $paramsIns = array(
                    "ID"               => $id,
                    "NRORG_ID"         => $medicao['NRORGID'],
                    "NRSEQCONTRATO"    => $medicao['NRSEQCONTRATO'],
                    "CDSERVORCA"       => $medicao['CDSERVORCA'],
                    "CDTPMEDIDASERV"   => $medicao['CDTPMEDIDAVARIAVEL'],
                    "PRODUCT_ID"       => $medicao['NREXTERNAL_PRODUCT_ID'],
                    "DTINICOMPET"      => $medicao['DTINICOMPET'],
                    "DTFINCOMPET"      => $medicao['DTFINCOMPET'],
                    "ERRO_COLETA"      => 'A medição de faturamento não foi calculada: Coleta não encontrada',
                    "IDTPCOLETA"       => $medicao['IDTPCOLETA'],
                    "IDTPERRO"         => 'W',
                    "SCRIPT_COUNT_PHP" => $scriptCountPHP,
                    "SCRIPT_LOG_PHP"   => $scriptLogPHP,
                    "SCRIPT_COUNT_SQL" => $scriptCountSQL,
                    "SCRIPT_LOG_SQL"   => $scriptLogSQL,
                    "NRORG"            => $nrorg
                );

                $this->integracaoFactory->insertErrosColetaOM($paramsIns);
            }
            else {
                $paramsUpdt = array(
                    "ERRO_COLETA"      => 'A medição de faturamento não foi calculada: Coleta não encontrada',
                    "IDTPCOLETA"       => $medicao['IDTPCOLETA'],
                    "IDTPERRO"         => 'W',
                    "SCRIPT_COUNT_PHP" => $scriptCountPHP,
                    "SCRIPT_LOG_PHP"   => $scriptLogPHP,
                    "SCRIPT_COUNT_SQL" => $scriptCountSQL,
                    "SCRIPT_LOG_SQL"   => $scriptLogSQL,
                    "NRORG"            => $nrorg, 
                    "NRSEQCONTRATO"    => $medicao['NRSEQCONTRATO'],
                    "CDSERVORCA"       => $medicao['CDSERVORCA'],
                    "CDTPMEDIDASERV"   => $medicao['CDTPMEDIDAVARIAVEL'],
                    "PRODUCT_ID"       => $medicao['NREXTERNAL_PRODUCT_ID'],
                    "DTINICOMPET"      => $medicao['DTINICOMPET'],
                    "DTFINCOMPET"      => $medicao['DTFINCOMPET']
                );

                $this->integracaoFactory->updateErrosColetaOM($paramsUpdt);
            }
        }
    }
    
}

?>